<?php

namespace ejfrancis;

use PDOException;

/**
 *                Brute Force Block class
 *
 *    Implementation by Evan Francis for use in AlpineAuth library, 2014.
 *  Inspired by work of Corey Ballou, http://stackoverflow.com/questions/2090910/how-can-i-throttle-user-login-attempts-in-php.
 *    MIT License http://opensource.org/licenses/MIT
 *
 * ========================    Setup      ===============================
 * 1) setup database connection in $_db array.
 * 1a. The 'auto_clear' option determines whether or not older database entries are cleared automatically
 * 2) (optional) set default throttle settings in $default_throttle_settings_array
 *
 * ==================== To Create MySQL Database ====================
 * CREATE TABLE IF NOT EXISTS `user_failed_logins` (
 * `id` int(11) NOT NULL AUTO_INCREMENT,
 * `user_id` bigint(20) NOT NULL,
 * `ip_address` int(11) unsigned DEFAULT NULL,
 * `attempted_at` datetime NOT NULL,
 * PRIMARY KEY (`id`)
 * ) ENGINE=InnoDB  DEFAULT CHARSET=latin1 AUTO_INCREMENT=4 ;
 *
 * ====================    Usage     ====================
 * === get login status. use this when building your login form ==
 * $BFBresponse = BruteForceBlock::getLoginStatus();
 * switch ($BFBresponse['status']){
 * case 'safe':
 * //safe to login
 * break;
 * case 'error':
 * //error occured. get message
 * $error_message = $BFBresponse['message'];
 * break;
 * case 'delay':
 * //time delay required before next login
 * $remaining_delay_in_seconds = $BFBresponse['message'];
 * break;
 * case 'captcha':
 * //captcha required
 * break;
 *
 * }
 *
 * == add a failed login attempt ==
 * $BFBresponse = BruteForceBlock::addFailedLoginAttempt($user_id, $ip_address);
 *
 * == clear the database ==
 * $BFBresponse = BruteForceBlock::clearDatabase();
 * if($BFBresponse !== true){
 * $error_message = $BFBresponse;
 * }
 */
//brute force block
class BruteForceBlock
{
    // array of throttle settings. # failed_attempts => response
    private static $default_throttle_settings = [
        50 => 2,            //delay in seconds
        150 => 4,            //delay in seconds
        300 => 'captcha'    //captcha
    ];

    //time frame to use when retrieving the number of recent failed logins from database
    private static $time_frame_minutes = 10;

    //add a failed login attempt to database. returns true, or error
    public static function addFailedLoginAttempt($user_id, $ip_address)
    {
        global $pdo;
        //get db connection
        //$db = BruteForceBlock::_databaseConnect();

        //attempt to insert failed login attempt
        if ($ip_address == null) return false;
        try {
            $stmt = $pdo->prepare('INSERT INTO nm_web_failed_logins SET user_id=?, ip_address = INET_ATON(?), attempted_at = NOW()');
            $stmt->bindParam(1, $user_id);
            $stmt->bindParam(2, $ip_address);
            $stmt->execute();
            //$result = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return true;
        } catch (PDOException $ex) {
            //return errors
            return $ex;
        }
    }

    //get the current login status. either safe, delay, catpcha, or error
    public static function getLoginStatus($options = null): array
    {
        global $pdo;
        //get db connection
        //$db = BruteForceBlock::_databaseConnect();

        //setup response array
        $response_array = array(
            'status' => 'safe',
            'message' => null
        );

        //attempt to retrieve latest failed login attempts
        $stmt = null;
        $row = null;
        $latest_failed_attempt_datetime = null;
        try {
            $stmt = $pdo->query('SELECT MAX(attempted_at) AS attempted_at FROM nm_web_failed_logins');
            $row = $stmt->fetch();
            //get latest attempt's timestamp
            $latest_failed_attempt_datetime = (int)date('U', strtotime($row['attempted_at']));
        } catch (PDOException $ex) {
            //return error
            $response_array['status'] = 'error';
            $response_array['message'] = $ex;
        }

        //get local var of throttle settings. check if options parameter set
        if ($options == null) {
            $throttle_settings = self::$default_throttle_settings;
        } else {
            //use options passed in
            $throttle_settings = $options;
        }
        //grab first throttle limit from key
        reset($throttle_settings);
        $first_throttle_limit = key($throttle_settings);

        //attempt to retrieve latest failed login attempts
        try {
            //get all failed attempst within time frame
            $get_number = $pdo->query('SELECT * FROM nm_web_failed_logins WHERE attempted_at > DATE_SUB(NOW(), INTERVAL ' . self::$time_frame_minutes . ' MINUTE)');
            $number_recent_failed = $get_number->rowCount();
            //reverse order of settings, for iteration
            krsort($throttle_settings);

            //if number of failed attempts is >= the minimum threshold in throttle_settings, react
            if ($number_recent_failed >= $first_throttle_limit) {
                //it's been decided the # of failed logins is troublesome. time to react accordingly, by checking throttle_settings
                foreach ($throttle_settings as $attempts => $delay) {
                    if ($number_recent_failed > $attempts) {
                        // we need to throttle based on delay
                        if (is_numeric($delay)) {
                            //find the time of the next allowed login
                            $next_login_minimum_time = $latest_failed_attempt_datetime + $delay;

                            //if the next allowed login time is in the future, calculate the remaining delay
                            if (time() < $next_login_minimum_time) {
                                $remaining_delay = $next_login_minimum_time - time();
                                // add status to response array
                                $response_array['status'] = 'delay';
                                $response_array['message'] = $remaining_delay;
                            } else {
                                // delay has been passed, safe to login
                                $response_array['status'] = 'safe';
                            }
                            //$remaining_delay = $delay - (time() - $latest_failed_attempt_datetime); //correct
                            //echo 'You must wait ' . $remaining_delay . ' seconds before your next login attempt';


                        } else {
                            // add status to response array
                            $response_array['status'] = 'captcha';
                        }
                        break;
                    }
                }

            }
            //clear database if config set
            /*if(self::$_db['auto_clear'] == true){
                //attempt to delete all records that are no longer recent/relevant
                try{
                    //get current timestamp
                    $now = date('Y-m-d H:i:s');
                    $stmt = $pdo->query('DELETE from user_failed_logins WHERE attempted_at < DATE_SUB(NOW(), INTERVAL '.(self::$time_frame_minutes * 2).' MINUTE)');
                    $stmt->execute();

                } catch(\PDOException $ex){
                    $response_array['status'] = 'error';
                    $response_array['message'] = $ex;
                }
            }*/

        } catch (PDOException $ex) {
            //return error
            $response_array['status'] = 'error';
            $response_array['message'] = $ex;
        }

        //return the response array containing status and message
        return $response_array;
    }

    //clear the database
    public static function clearDatabase()
    {
        global $pdo;
        //get db connection
        //$db = BruteForceBlock::_databaseConnect();

        //attempt to delete all records
        try {
            $pdo->query('DELETE FROM nm_web_failed_logins');
            return true;
        } catch (PDOException $ex) {
            //return errors
            return $ex;
        }
    }
}