<?php

namespace networkmanager\storage;

use IteratorIterator;
use PDO;
use PDOException;

class ServersDao
{
    function getServersTotal(): int
    {
        global $pdo;
        try {
            return $pdo->query('SELECT COUNT(*) FROM nm_servers;')->fetchColumn();
        } catch (PDOException $ex) {
            return 0;
        }
    }

    function getServers($limit, $offset): ?IteratorIterator
    {
        global $pdo;
        if ($offset < 0) $offset = 0;
        try {
            $statement = $pdo->prepare('SELECT id, servername, displayname, ip, port, motd, restricted, online FROM nm_servers LIMIT ? OFFSET ?;');
            $statement->bindParam(1, $limit, PDO::PARAM_INT);
            $statement->bindParam(2, $offset, PDO::PARAM_INT);
            $statement->execute();
            $statement->setFetchMode(PDO::FETCH_ASSOC);
            return new IteratorIterator($statement);
        } catch (PDOException $ex) {
            return null;
        }
    }

    function getServerGroupsTotal(): int
    {
        global $pdo;
        try {
            return $pdo->query('SELECT COUNT(*) FROM nm_server_groups;')->fetchColumn();
        } catch (PDOException $ex) {
            return 0;
        }
    }

    function getServerGroups($limit, $offset): ?IteratorIterator
    {
        global $pdo;
        if ($offset < 0) $offset = 0;
        try {
            $statement = $pdo->prepare('SELECT id, groupname, servers FROM nm_server_groups ORDER BY id LIMIT ? OFFSET ?;');
            $statement->bindParam(1, $limit, PDO::PARAM_INT);
            $statement->bindParam(2, $offset, PDO::PARAM_INT);
            $statement->execute();
            $statement->setFetchMode(PDO::FETCH_ASSOC);
            return new IteratorIterator($statement);
        } catch (PDOException $ex) {
            return null;
        }
    }

    function updateServer($id, $servername, $displayName, $ip, $port, $motd, $allowedVersions): bool
    {
        global $pdo;
        try {
            $statement = $pdo->prepare('UPDATE nm_servers SET `servername`=?, `displayname`=?, `ip`=?, `port`=?, `motd`=?, `allowed_versions`=? WHERE `id`=?;');
            $statement->bindParam(1, $servername);
            $statement->bindParam(2, $displayName);
            $statement->bindParam(3, $ip);
            $statement->bindParam(4, $port, PDO::PARAM_INT);
            $statement->bindParam(5, $motd);
            $statement->bindParam(6, $allowedVersions);
            $statement->bindParam(7, $id, PDO::PARAM_INT);
            return $statement->execute();
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return false;
        }
    }

    function setServerRestricted($id, $restricted): bool
    {
        global $pdo;
        try {
            $statement = $pdo->prepare('UPDATE nm_servers SET `restricted`=? WHERE `id`=?;');
            $statement->bindParam(1, $restricted, PDO::PARAM_INT);
            $statement->bindParam(2, $id, PDO::PARAM_INT);
            return $statement->execute();
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return false;
        }
    }

    function deleteServer($id): bool
    {
        global $pdo;
        try {
            $statement = $pdo->prepare('DELETE FROM nm_servers WHERE `id`=?;');
            $statement->bindParam(1, $id, PDO::PARAM_INT);
            return $statement->execute();
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return false;
        }
    }

    function getServerNameById($id): ?string
    {
        global $pdo;
        try {
            $statement = $pdo->prepare('SELECT `servername` FROM nm_servers WHERE `id`=?;');
            $statement->bindParam(1, $id, PDO::PARAM_INT);
            $statement->execute();
            return $statement->fetch()['servername'];
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return null;
        }
    }

    function getServerIdByName($name): ?int
    {
        global $pdo;
        if (is_numeric($name)) {
            return (int)$name;
        }
        try {
            $statement = $pdo->prepare('SELECT `id` FROM nm_servers WHERE `servername`=?;');
            $statement->bindParam(1, $name);
            $statement->execute();
            return (int)$statement->fetch()['id'];
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return null;
        }
    }

    function serverNameExists($serverName): bool
    {
        global $pdo;
        try {
            $statement = $pdo->prepare('SELECT `id` FROM nm_servers WHERE `servername`=?;');
            $statement->bindParam(1, $serverName);
            $statement->execute();
            return $statement->rowCount() > 0;
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return false;
        }
    }

    function insertServer($serverName, $displayName, $ip, $port, $motd, $allowedVersions, $restricted): bool
    {
        global $pdo;
        try {
            $statement = $pdo->prepare('INSERT INTO nm_servers(servername, displayname, ip, port, motd, allowed_versions, restricted) VALUES (?, ?, ?, ?, ?, ?, ?);');
            $statement->bindParam(1, $serverName);
            $statement->bindParam(2, $displayName);
            $statement->bindParam(3, $ip);
            $statement->bindParam(4, $port, PDO::PARAM_INT);
            $statement->bindParam(5, $motd);
            $statement->bindParam(6, $allowedVersions);
            $statement->bindParam(7, $restricted, PDO::PARAM_BOOL);
            return $statement->execute();
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return false;
        }
    }

    function getServerGroupsById($id, $limit, $offset): ?array
    {
        global $pdo;
        if ($offset < 0) $offset = 0;
        try {
            $statement = $pdo->prepare('SELECT id, groupname, servers FROM nm_server_groups WHERE `id`=? ORDER BY id LIMIT ? OFFSET ?;');
            $statement->bindParam(1, $id, PDO::PARAM_INT);
            $statement->bindParam(2, $limit, PDO::PARAM_INT);
            $statement->bindParam(3, $offset, PDO::PARAM_INT);
            $statement->execute();
            if ($result = $statement->fetch(PDO::FETCH_ASSOC)) {
                return $result;
            } else {
                return null;
            }
        } catch (PDOException $ex) {
            return null;
        }
    }

    function getServersFromGroupById($id): string
    {
        global $pdo;
        try {
            $statement = $pdo->prepare('SELECT `servers` FROM nm_server_groups WHERE `id`=?;');
            $statement->bindParam(1, $id, PDO::PARAM_INT);
            $statement->execute();
            return $statement->fetch()['servers'];
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return '[]';
        }
    }

    function serverGroupNameExists($serverGroupName): bool
    {
        global $pdo;
        try {
            $statement = $pdo->prepare('SELECT `id` FROM nm_server_groups WHERE `groupname`=?');
            $statement->bindParam(1, $serverGroupName);
            $statement->execute();
            return $statement->rowCount() > 0;
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return false;
        }
    }

    function insertServerGroup($serverGroupName, $servers): bool
    {
        global $pdo;
        try {
            $statement = $pdo->prepare('INSERT INTO nm_server_groups(`groupname`, `servers`) VALUES (?, ?);');
            $statement->bindParam(1, $serverGroupName);
            $statement->bindValue(2, '[' . $servers . ']');
            return $statement->execute();
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return false;
        }
    }

    function deleteServerGroup($id): bool {
        global $pdo;
        try {
            $statement = $pdo->prepare('DELETE FROM nm_server_groups WHERE `id`=?;');
            $statement->bindParam(1, $id, PDO::PARAM_INT);
            return $statement->execute();
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return false;
        }
    }

    function updateServersInServerGroup($groupId, $servers): bool
    {
        global $pdo;
        try {
            $statement = $pdo->prepare('UPDATE nm_server_groups SET `servers`=? WHERE `id`=?;');
            $statement->bindParam(1, $servers);
            $statement->bindParam(2, $groupId, 1);
            return $statement->execute();
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return false;
        }
    }

    function getTableColumns(): ?array {
        global $pdo;
        try {
            $statement = $pdo->query('SHOW COLUMNS FROM nm_servers;');
            if (!$statement) return null;
            if ($result = $statement->fetchAll()) {
                return $result;
            } else {
                return null;
            }
        } catch (PDOException $ex) {
            errorAlert($ex->getMessage());
            return null;
        }
    }
}